from PyQt4 import QtCore, QtGui
import utils
import pydosh_rc

class MultiComboBox(QtGui.QComboBox):
	""" Extension of QComboBox widget that allows for multiple selection
	"""
	# Signal containing list of selected indexes
	selectionChanged = QtCore.pyqtSignal('PyQt_PyObject')

	def __init__(self, parent=None):
		super(MultiComboBox, self).__init__(parent=parent)

		self._defaultText = ''
		self.__persistDropdown = False

		self.setLineEdit(QtGui.QLineEdit())
		self.setInsertPolicy(QtGui.QComboBox.NoInsert)
		self.lineEdit().blockSignals(True)

		self.view().installEventFilter(self)
		self.view().window().installEventFilter(self)
		self.view().viewport().installEventFilter(self)
		self.installEventFilter(self)

		self.activated[int].connect(self.__toggleCheckState)

	def reset(self):
		""" Reset the model and clear selection if necessary
		"""
		itemsBefore = [index.data().toString() for index in self.__checkedItems()]

		self.model().select()

		itemsAfter = [index.data().toString() for index in self.__checkedItems()]

		if itemsBefore != itemsAfter:
			self.clearAll()

	def setModel(self, model):
		super(MultiComboBox, self).setModel(model)
		# This is a bit odd, but seem to need it
		self.setModelColumn(self.modelColumn())
		model.rowsInserted.connect(self.__updateCheckedItems)
		model.rowsRemoved.connect(self.__updateCheckedItems)
		model.dataChanged.connect(self.__updateCheckedItems)
		self.__updateCheckedItems()

	def dataChanged(self):
		""" Indicates if the underlying data has been updated. 
		"""
		self.model().select()

	def __checkAll(self, check=False):
		""" Function to set all the items as checked or unchecked based on the argument.

			Args:
				check(bool): check state
		"""
		searchState = QtCore.Qt.Unchecked if check else QtCore.Qt.Checked
		assignState = QtCore.Qt.Checked if check else QtCore.Qt.Unchecked

		modelIndex = self.model().index(0, self.modelColumn(), self.rootModelIndex())
		modelIndexList = self.model().match(modelIndex, QtCore.Qt.CheckStateRole, searchState, -1, QtCore.Qt.MatchExactly)

		with utils.signalsBlocked(self.model()):
			for index in modelIndexList:
				self.setItemData(index.row(), assignState, QtCore.Qt.CheckStateRole)

		self.__updateCheckedItems()

	def setLineEdit(self, edit):
		edit.setReadOnly(True)
		edit.installEventFilter(self)
		super(MultiComboBox, self).setLineEdit(edit)

	def defaultText(self):
		""" Function to get the default text of ComboBox.

			 Returns:
				(str) return the default text.
		"""
		return self._defaultText

	def setDefaultText(self, text):
		""" Function to set the default text of ComboBox.

			Args:
				text (str): string for default text
		"""
		self._defaultText = text
		self.__updateCheckedItems()

	def clearAll(self):
		""" Event for clear all check from the list view. 
		"""
		self.__checkAll(False)

	def checkAll(self):
		""" Event for check all from the list view.
		"""
		self.__checkAll(True)

	def eventFilter(self, receiver, event):
		if event.type() in (QtCore.QEvent.KeyPress, QtCore.QEvent.KeyRelease):
			if receiver == self and event.key() in (QtCore.Qt.Key_Up, QtCore.Qt.Key_Down):
				self.showPopup()
				return True

			elif event.key() in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return, QtCore.Qt.Key_Escape):
				super(MultiComboBox, self).hidePopup()
				return True

		elif event.type() == QtCore.QEvent.MouseButtonPress:
			# Cancel the dropdown persist if we've clicked outside the widget
			if receiver == self.view().window():
				self.__persistDropdown = False

		return False

	def __toggleCheckState(self, index):
		value = self.itemData(index, QtCore.Qt.CheckStateRole)

		if value.isValid():
			if value.toPyObject() == QtCore.Qt.Unchecked:
				state = QtCore.Qt.Checked
			else:
				state = QtCore.Qt.Unchecked
			self.setItemData(index, state, QtCore.Qt.CheckStateRole)

	def __checkedItems(self):
		indexes = []
		if self.model():
			currentIndex = self.model().index(0, self.modelColumn(), self.rootModelIndex())
			indexes = self.model().match(currentIndex, QtCore.Qt.CheckStateRole, QtCore.Qt.Checked, -1, QtCore.Qt.MatchExactly)
		return indexes

	def contextMenuEvent(self, event):
		""" Event for create context menu.
			Args:
				event (QEvent): mouse right click event.
		"""
		quitAction = QtGui.QAction('&Quit', self)
		quitAction.setShortcut('Alt+q')
		quitAction.setStatusTip('Exit the program')
		quitAction.setIcon(QtGui.QIcon(':/icons/exit.png'))

		signalMap = {}
		contextMenu = QtGui.QMenu()

		clearAllAction = contextMenu.addAction(QtGui.QIcon(':/icons/cross.png'), 'Clear All')
		clearAllAction.setIconVisibleInMenu(True)
		signalMap[clearAllAction] = self.clearAll
		checkAllAction = contextMenu.addAction(QtGui.QIcon(':/icons/tick.png'), 'Check All')
		checkAllAction.setIconVisibleInMenu(True)
		signalMap[checkAllAction] = self.checkAll

		contextMenuAction = contextMenu.exec_(QtGui.QCursor.pos())
		if contextMenuAction and signalMap.has_key(contextMenuAction):
			signalMap[contextMenuAction]()

	def __updateCheckedItems(self):
		items = self.__checkedItems()
		if items:
			self.setEditText(QtCore.QStringList([index.data().toString() for index in items]).join(', '))
		else:
			self.setEditText(self._defaultText)

		self.selectionChanged.emit(items)

	def showPopup(self):
		self.__persistDropdown = QtGui.QApplication.keyboardModifiers() == QtCore.Qt.ShiftModifier
		super(MultiComboBox, self).showPopup()

	def hidePopup(self):
		if not self.__persistDropdown:
			super(MultiComboBox, self).hidePopup()


class MyCombo(QtGui.QComboBox):
	def __init__(self, parent=None):
		super(MyCombo, self).__init__(parent=parent)

	def paintEvent(self, event):
		painter = QtGui.QStylePainter(self)
		opt = QtGui.QStyleOptionComboBox()
		self.initStyleOption(opt)
		#painter.drawComplexControl(QtGui.QStyle.CC_ComboBox, opt)
		#painter.drawControl(QtGui.QStyle.CE_ComboBoxLabel, opt)
#		super(MyCombo, self).paintEvent(event)



def main():
	from searchlineedit import SearchLineEdit
#	from  signaltracer import SignalTracer
#	tracer = SignalTracer()
	app = QtGui.QApplication(sys.argv)
	#app.setStyle(QtGui.QStyleFactory.create("Plastique"))
	#pdb.set_trace()
	#lineEdit = SearchLineEdit()
	combo = MyCombo()
	combo.setFrame(False)
#	combo.setStyleSheet ("QComboBox::drop-down {border-width: 0px;} QComboBox::down-arrow {image: url(noimg); border-width: 0px;}")
	#tracer.monitor(lineEdit, widget, lineEdit.clearButton)

	#widget.setLineEdit(lineEdit)
	#lineEdit.clearButtonPressed.connect(widget.clearAll)
	label = QtGui.QLabel('releaseTime')
	combo.addItem(QtGui.QIcon('in.png'), '')
	combo.addItem(QtGui.QIcon('ca.png'), '')
	combo.addItem(QtGui.QIcon('gb.png'), '')
	combo.addItem(QtGui.QIcon('us.png'), '')
	#widget.addItems(['item %d' %i for i in xrange(2)])
#	widget.setDefaultText('all')

	dialog = QtGui.QDialog()
	layout = QtGui.QVBoxLayout()
	hLayout = QtGui.QHBoxLayout()
	hLayout.addWidget(label)
	hLayout.addWidget(combo)
	layout.addLayout(hLayout)

	dialog.setLayout(layout)
	dialog.show()

	app.exec_()	
	return 0



if __name__ == '__main__':
	import sys
	sys.exit(main())




kRecordColumn_RecordId = 0
kRecordColumn_Checked = 1
kRecordColumn_Tags = 2
kRecordColumn_CheckDate = 3
kRecordColumn_Date = 4
kRecordColumn_AccountTypeId = 5
kRecordColumn_AccountTypeName = 6
kRecordColumn_Description = 7
kRecordColumn_Txdate = 8
kRecordColumn_Amount = 9
kRecordColumn_InsertDate = 10
kRecordColumn_RawData = 11

kImportColumn_Status = 0
kImportColumn_Date = 1
kImportColumn_TxDate = 2
kImportColumn_Credit = 3
kImportColumn_Debit = 4
kImportColumn_Description = 5 

kDate_All = 0
kDate_PreviousYear = 1
kDate_PreviousMonth = 2
kdate_LastImport =3

kCheckedStatus_All =0
kCheckedStatus_Checked = 1
kCheckedStatus_UnChecked = 2

kInOutStatus_All = 0
kInOutStatus_In = 1
kInOutStatus_Out = 2

kTagCombo_All = 0
kTagCombo_With = 1
kTagCombo_Without = 2

kAccountTypeColumn_AccountTypeId = 0
kAccountTypeColumn_AccountName = 1
kAccountTypeColumn_DateField = 2
kAccountTypeColumn_DescriptionField = 3
kAccountTypeColumn_CreditField = 4
kAccountTypeColumn_DebitField = 5
kAccountTypeColumn_CurrencySign = 6
kAccountTypeColumn_DateFormat = 7

kTagsColumn_TagId = 0
kTagsColumn_TagName = 1
kTagsColumn_RecordIds = 2
kTagsColumn_Amount_in = 3
kTagsColumn_Amount_out = 4

kRecordTagsColumn_RecordTagId = 0
kRecordTagsColumn_RecordId = 1
kRecordTagsColumn_TagId = 2



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/pydosh.ui'
#
# Created: Sat Jun 22 12:53:39 2013
#      by: PyQt4 UI code generator 4.10
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_pydosh(object):
    def setupUi(self, pydosh):
        pydosh.setObjectName(_fromUtf8("pydosh"))
        pydosh.resize(1092, 745)
        self.widget_6 = QtGui.QWidget(pydosh)
        self.widget_6.setObjectName(_fromUtf8("widget_6"))
        self.gridLayout_4 = QtGui.QGridLayout(self.widget_6)
        self.gridLayout_4.setObjectName(_fromUtf8("gridLayout_4"))
        self.widget_3 = QtGui.QWidget(self.widget_6)
        self.widget_3.setObjectName(_fromUtf8("widget_3"))
        self.gridLayout_3 = QtGui.QGridLayout(self.widget_3)
        self.gridLayout_3.setMargin(0)
        self.gridLayout_3.setObjectName(_fromUtf8("gridLayout_3"))
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 0, 3, 1, 1)
        self.groupBox_2 = QtGui.QGroupBox(self.widget_3)
        self.groupBox_2.setFlat(False)
        self.groupBox_2.setCheckable(False)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.gridLayout_5 = QtGui.QGridLayout(self.groupBox_2)
        self.gridLayout_5.setObjectName(_fromUtf8("gridLayout_5"))
        self.label = QtGui.QLabel(self.groupBox_2)
        self.label.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout_5.addWidget(self.label, 0, 0, 1, 1)
        self.endDateEdit = QtGui.QDateEdit(self.groupBox_2)
        self.endDateEdit.setObjectName(_fromUtf8("endDateEdit"))
        self.gridLayout_5.addWidget(self.endDateEdit, 0, 1, 1, 2)
        self.label_2 = QtGui.QLabel(self.groupBox_2)
        self.label_2.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout_5.addWidget(self.label_2, 1, 0, 1, 1)
        self.startDateEdit = QtGui.QDateEdit(self.groupBox_2)
        self.startDateEdit.setObjectName(_fromUtf8("startDateEdit"))
        self.gridLayout_5.addWidget(self.startDateEdit, 1, 1, 1, 2)
        self.dateCombo = QtGui.QComboBox(self.groupBox_2)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dateCombo.sizePolicy().hasHeightForWidth())
        self.dateCombo.setSizePolicy(sizePolicy)
        self.dateCombo.setObjectName(_fromUtf8("dateCombo"))
        self.gridLayout_5.addWidget(self.dateCombo, 2, 1, 1, 2)
        self.gridLayout_3.addWidget(self.groupBox_2, 0, 0, 1, 1)
        self.groupBox = QtGui.QGroupBox(self.widget_3)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.gridLayout = QtGui.QGridLayout(self.groupBox)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label_14 = QtGui.QLabel(self.groupBox)
        self.label_14.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_14.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.gridLayout.addWidget(self.label_14, 2, 2, 1, 1)
        self.label_6 = QtGui.QLabel(self.groupBox)
        self.label_6.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.gridLayout.addWidget(self.label_6, 3, 2, 1, 1)
        self.descEdit = SearchLineEdit(self.groupBox)
        self.descEdit.setObjectName(_fromUtf8("descEdit"))
        self.gridLayout.addWidget(self.descEdit, 2, 3, 1, 1)
        self.inoutCombo = QtGui.QComboBox(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.inoutCombo.sizePolicy().hasHeightForWidth())
        self.inoutCombo.setSizePolicy(sizePolicy)
        self.inoutCombo.setObjectName(_fromUtf8("inoutCombo"))
        self.gridLayout.addWidget(self.inoutCombo, 2, 1, 1, 1)
        self.label_13 = QtGui.QLabel(self.groupBox)
        self.label_13.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_13.setObjectName(_fromUtf8("label_13"))
        self.gridLayout.addWidget(self.label_13, 2, 0, 1, 1)
        self.accountCombo = MultiComboBox(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.accountCombo.sizePolicy().hasHeightForWidth())
        self.accountCombo.setSizePolicy(sizePolicy)
        self.accountCombo.setObjectName(_fromUtf8("accountCombo"))
        self.gridLayout.addWidget(self.accountCombo, 1, 3, 1, 1)
        self.label_4 = QtGui.QLabel(self.groupBox)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.gridLayout.addWidget(self.label_4, 1, 2, 1, 1)
        self.label_15 = QtGui.QLabel(self.groupBox)
        self.label_15.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_15.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_15.setObjectName(_fromUtf8("label_15"))
        self.gridLayout.addWidget(self.label_15, 3, 0, 1, 1)
        self.tagsCombo = QtGui.QComboBox(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tagsCombo.sizePolicy().hasHeightForWidth())
        self.tagsCombo.setSizePolicy(sizePolicy)
        self.tagsCombo.setObjectName(_fromUtf8("tagsCombo"))
        self.gridLayout.addWidget(self.tagsCombo, 3, 1, 1, 1)
        self.checkedCombo = QtGui.QComboBox(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.checkedCombo.sizePolicy().hasHeightForWidth())
        self.checkedCombo.setSizePolicy(sizePolicy)
        self.checkedCombo.setObjectName(_fromUtf8("checkedCombo"))
        self.gridLayout.addWidget(self.checkedCombo, 1, 1, 1, 1)
        self.label_3 = QtGui.QLabel(self.groupBox)
        self.label_3.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 1, 0, 1, 1)
        self.amountEdit = SearchLineEdit(self.groupBox)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.amountEdit.sizePolicy().hasHeightForWidth())
        self.amountEdit.setSizePolicy(sizePolicy)
        self.amountEdit.setObjectName(_fromUtf8("amountEdit"))
        self.gridLayout.addWidget(self.amountEdit, 3, 3, 1, 1)
        self.gridLayout_3.addWidget(self.groupBox, 0, 2, 1, 1)
        self.gridLayout_4.addWidget(self.widget_3, 0, 0, 1, 1)
        self.widget_4 = QtGui.QWidget(self.widget_6)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget_4.sizePolicy().hasHeightForWidth())
        self.widget_4.setSizePolicy(sizePolicy)
        self.widget_4.setObjectName(_fromUtf8("widget_4"))
        self.horizontalLayout_4 = QtGui.QHBoxLayout(self.widget_4)
        self.horizontalLayout_4.setMargin(0)
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.groupBox_4 = QtGui.QGroupBox(self.widget_4)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_4.sizePolicy().hasHeightForWidth())
        self.groupBox_4.setSizePolicy(sizePolicy)
        self.groupBox_4.setObjectName(_fromUtf8("groupBox_4"))
        self._2 = QtGui.QVBoxLayout(self.groupBox_4)
        self._2.setObjectName(_fromUtf8("_2"))
        self.widget = QtGui.QWidget(self.groupBox_4)
        self.widget.setObjectName(_fromUtf8("widget"))
        self._4 = QtGui.QHBoxLayout(self.widget)
        self._4.setContentsMargins(0, -1, 0, -1)
        self._4.setObjectName(_fromUtf8("_4"))
        self.toggleCheckButton = QtGui.QToolButton(self.widget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/accept.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toggleCheckButton.setIcon(icon)
        self.toggleCheckButton.setObjectName(_fromUtf8("toggleCheckButton"))
        self._4.addWidget(self.toggleCheckButton)
        self.reloadButton = QtGui.QToolButton(self.widget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/arrow_refresh.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.reloadButton.setIcon(icon1)
        self.reloadButton.setObjectName(_fromUtf8("reloadButton"))
        self._4.addWidget(self.reloadButton)
        self.deleteButton = QtGui.QToolButton(self.widget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/cross.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.deleteButton.setIcon(icon2)
        self.deleteButton.setObjectName(_fromUtf8("deleteButton"))
        self._4.addWidget(self.deleteButton)
        spacerItem1 = QtGui.QSpacerItem(703, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self._4.addItem(spacerItem1)
        self.label_7 = QtGui.QLabel(self.widget)
        self.label_7.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_7.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_7.setObjectName(_fromUtf8("label_7"))
        self._4.addWidget(self.label_7)
        self.scrolltoEdit = SearchLineEdit(self.widget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrolltoEdit.sizePolicy().hasHeightForWidth())
        self.scrolltoEdit.setSizePolicy(sizePolicy)
        self.scrolltoEdit.setObjectName(_fromUtf8("scrolltoEdit"))
        self._4.addWidget(self.scrolltoEdit)
        self._2.addWidget(self.widget)
        self.tableView = QtGui.QTableView(self.groupBox_4)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tableView.sizePolicy().hasHeightForWidth())
        self.tableView.setSizePolicy(sizePolicy)
        self.tableView.setObjectName(_fromUtf8("tableView"))
        self._2.addWidget(self.tableView)
        self.widget_5 = QtGui.QWidget(self.groupBox_4)
        self.widget_5.setObjectName(_fromUtf8("widget_5"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.widget_5)
        self.horizontalLayout_3.setMargin(0)
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.connectionStatusText = QtGui.QLabel(self.widget_5)
        self.connectionStatusText.setObjectName(_fromUtf8("connectionStatusText"))
        self.horizontalLayout_3.addWidget(self.connectionStatusText)
        spacerItem2 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.label_10 = QtGui.QLabel(self.widget_5)
        self.label_10.setObjectName(_fromUtf8("label_10"))
        self.horizontalLayout_3.addWidget(self.label_10)
        self.inTotalLabel = QtGui.QLabel(self.widget_5)
        self.inTotalLabel.setLineWidth(-1)
        self.inTotalLabel.setObjectName(_fromUtf8("inTotalLabel"))
        self.horizontalLayout_3.addWidget(self.inTotalLabel)
        spacerItem3 = QtGui.QSpacerItem(28, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.label_11 = QtGui.QLabel(self.widget_5)
        self.label_11.setObjectName(_fromUtf8("label_11"))
        self.horizontalLayout_3.addWidget(self.label_11)
        self.outTotalLabel = QtGui.QLabel(self.widget_5)
        self.outTotalLabel.setObjectName(_fromUtf8("outTotalLabel"))
        self.horizontalLayout_3.addWidget(self.outTotalLabel)
        spacerItem4 = QtGui.QSpacerItem(134, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem4)
        self.label_12 = QtGui.QLabel(self.widget_5)
        self.label_12.setObjectName(_fromUtf8("label_12"))
        self.horizontalLayout_3.addWidget(self.label_12)
        self.recordCountLabel = QtGui.QLabel(self.widget_5)
        self.recordCountLabel.setObjectName(_fromUtf8("recordCountLabel"))
        self.horizontalLayout_3.addWidget(self.recordCountLabel)
        self._2.addWidget(self.widget_5)
        self.horizontalLayout_4.addWidget(self.groupBox_4)
        self.groupBox_3 = QtGui.QGroupBox(self.widget_4)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setObjectName(_fromUtf8("groupBox_3"))
        self._3 = QtGui.QVBoxLayout(self.groupBox_3)
        self._3.setObjectName(_fromUtf8("_3"))
        self.widget_2 = QtGui.QWidget(self.groupBox_3)
        self.widget_2.setObjectName(_fromUtf8("widget_2"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.widget_2)
        self.horizontalLayout.setContentsMargins(0, -1, -1, -1)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.addTagButton = QtGui.QToolButton(self.widget_2)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/add.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.addTagButton.setIcon(icon3)
        self.addTagButton.setObjectName(_fromUtf8("addTagButton"))
        self.horizontalLayout.addWidget(self.addTagButton)
        self.removeTagButton = QtGui.QToolButton(self.widget_2)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/delete.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.removeTagButton.setIcon(icon4)
        self.removeTagButton.setObjectName(_fromUtf8("removeTagButton"))
        self.horizontalLayout.addWidget(self.removeTagButton)
        spacerItem5 = QtGui.QSpacerItem(60, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self._3.addWidget(self.widget_2)
        self.tagView = TagTableView(self.groupBox_3)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tagView.sizePolicy().hasHeightForWidth())
        self.tagView.setSizePolicy(sizePolicy)
        self.tagView.setObjectName(_fromUtf8("tagView"))
        self._3.addWidget(self.tagView)
        self.horizontalLayout_4.addWidget(self.groupBox_3)
        self.gridLayout_4.addWidget(self.widget_4, 1, 0, 1, 1)
        pydosh.setCentralWidget(self.widget_6)
        self.label_14.setBuddy(self.descEdit)
        self.label_6.setBuddy(self.amountEdit)
        self.label_7.setBuddy(self.scrolltoEdit)

        self.retranslateUi(pydosh)
        QtCore.QMetaObject.connectSlotsByName(pydosh)

    def retranslateUi(self, pydosh):
        pydosh.setWindowTitle(_translate("pydosh", "pydosh", None))
        self.groupBox_2.setTitle(_translate("pydosh", "date", None))
        self.label.setText(_translate("pydosh", "end", None))
        self.endDateEdit.setToolTip(_translate("pydosh", "<html><head/><body><p>Set the end date filter</p></body></html>", None))
        self.label_2.setText(_translate("pydosh", "start", None))
        self.startDateEdit.setToolTip(_translate("pydosh", "<html><head/><body><p>Set the start date filter</p></body></html>", None))
        self.dateCombo.setToolTip(_translate("pydosh", "<html><head/><body><p>Select a date filter</p></body></html>", None))
        self.groupBox.setTitle(_translate("pydosh", "filter", None))
        self.label_14.setText(_translate("pydosh", "&description", None))
        self.label_6.setText(_translate("pydosh", "&amount", None))
        self.descEdit.setToolTip(_translate("pydosh", "<html><head/><body><p>filter on description (Alt-d)</p></body></html>", None))
        self.inoutCombo.setToolTip(_translate("pydosh", "<html><head/><body><p>filter on money in or out</p></body></html>", None))
        self.label_13.setText(_translate("pydosh", "in/out", None))
        self.accountCombo.setToolTip(_translate("pydosh", "<html><head/><body><p>filter on account type</p></body></html>", None))
        self.label_4.setText(_translate("pydosh", "account", None))
        self.label_15.setText(_translate("pydosh", "tags", None))
        self.tagsCombo.setToolTip(_translate("pydosh", "filter on checked status", None))
        self.checkedCombo.setToolTip(_translate("pydosh", "filter on checked status", None))
        self.label_3.setText(_translate("pydosh", "checked", None))
        self.amountEdit.setToolTip(_translate("pydosh", "<html><head/><body><p>filter by amount (Alt-a)</p><p>Accepts operators = &gt; &gt;= &lt; &lt;=</p></body></html>", None))
        self.groupBox_4.setTitle(_translate("pydosh", "Records", None))
        self.toggleCheckButton.setToolTip(_translate("pydosh", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'Arial\'; font-size:10pt;\">toggle checked</span></p></body></html>", None))
        self.toggleCheckButton.setText(_translate("pydosh", "&t", None))
        self.reloadButton.setToolTip(_translate("pydosh", "reload data (Alt-r)", None))
        self.reloadButton.setText(_translate("pydosh", "&r", None))
        self.deleteButton.setToolTip(_translate("pydosh", "<html><head/><body><p>delete record(s)</p></body></html>", None))
        self.deleteButton.setText(_translate("pydosh", "&r", None))
        self.label_7.setText(_translate("pydosh", "&find", None))
        self.scrolltoEdit.setToolTip(_translate("pydosh", "<html><head/><body><p>find records matching description </p></body></html>", None))
        self.connectionStatusText.setText(_translate("pydosh", "TextLabel", None))
        self.label_10.setText(_translate("pydosh", "in:", None))
        self.inTotalLabel.setToolTip(_translate("pydosh", "total incoming for filtered records", None))
        self.inTotalLabel.setText(_translate("pydosh", "TextLabel", None))
        self.label_11.setText(_translate("pydosh", "out:", None))
        self.outTotalLabel.setToolTip(_translate("pydosh", "total outgoing for filtered records", None))
        self.outTotalLabel.setText(_translate("pydosh", "TextLabel", None))
        self.label_12.setText(_translate("pydosh", "showing:", None))
        self.recordCountLabel.setToolTip(_translate("pydosh", "filtered records / total records", None))
        self.recordCountLabel.setText(_translate("pydosh", "TextLabel", None))
        self.groupBox_3.setTitle(_translate("pydosh", "Tags", None))
        self.addTagButton.setText(_translate("pydosh", "...", None))
        self.removeTagButton.setText(_translate("pydosh", "...", None))

from searchlineedit import SearchLineEdit
from multicombobox import MultiComboBox
from views import TagTableView
import pydosh_rc



import re
from PyQt4 import QtCore, QtGui, QtSql
from ui_settings import Ui_Settings
from ui_login import Ui_Login
from ui_import import Ui_Import
import enum
import utils
from database import db, DatabaseNotInitialisedException, ConnectionException
from delegates import AccountDelegate
from models import AccountEditModel, ImportModel

import csv

def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, **kwargs):
	# csv.py doesn't do Unicode; encode temporarily as UTF-8:
	csv_reader = csv.reader(utf_8_encoder(unicode_csv_data),
						dialect=dialect, **kwargs)
	for row in csv_reader:
		# decode UTF-8 back to Unicode, cell by cell:
		yield [unicode(cell, 'utf-8') for cell in row]

def utf_8_encoder(unicode_csv_data):
	for line in unicode_csv_data:
		yield line.encode('utf-8')

class UserCancelledException(Exception):
	""" Exception to indicate user has cancelled the current operation
	"""

class DecoderError(Exception):
	""" General Decoder exceptions
	"""

class ImportDialog(Ui_Import, QtGui.QDialog):
	def __init__(self, files, parent=None):
		super(ImportDialog, self).__init__(parent=parent)
		self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
		self.setupUi(self)
		self.__rawData = {}
		self.__importInProgress = False
		self.__cancelImport = False
		self.__accountsModel = None

		self.progressBar.setVisible(False)

		accountModel = QtSql.QSqlTableModel(self)
		accountModel.setTable('accounttypes')
		accountModel.select()

		self.accountTypeComboBox.setModel(accountModel)
		self.accountTypeComboBox.setModelColumn(enum.kAccountTypeColumn_AccountName)
		self.accountTypeComboBox.setCurrentIndex(-1)

		self.__accountsModel = accountModel

		model = QtGui.QStandardItemModel()

		for filename in files:
			csvfile = QtCore.QFile(filename)

			if not csvfile.open(QtCore.QIODevice.ReadOnly | QtCore.QIODevice.Text):
				raise Exception('Cannot open file %r' % filename)

			while not csvfile.atEnd():
				rawdata = csvfile.readLine().trimmed()
				dataDict = self.__rawData.setdefault(filename, [])
				dataDict.append(rawdata)

				row =  unicode_csv_reader([rawdata.data().decode('utf8')]).next()
				items = [QtGui.QStandardItem(item) for item in row]
				model.appendRow(items)

		self.importCancelButton.setEnabled(False)
		self.selectAllButton.setEnabled(False)

		self.view.setModel(model)
		self.view.resizeColumnsToContents()

		self.accountTypeComboBox.currentIndexChanged.connect(self.setAccountType)
		self.importCancelButton.clicked.connect(self.__importCancelPressed)
		self.selectAllButton.clicked.connect(self.view.selectAll)
		self.closeButton.clicked.connect(self.__close)

	def setAccountType(self, index):
		""" Account selection has changed

			Get settings for the account and create new model to decode the data
		"""
		dateField = self.__accountsModel.index(index, enum.kAccountTypeColumn_DateField).data().toPyObject()
		descriptionField = self.__accountsModel.index(index, enum.kAccountTypeColumn_DescriptionField).data().toPyObject()
		creditField = self.__accountsModel.index(index, enum.kAccountTypeColumn_CreditField).data().toPyObject()
		debitField = self.__accountsModel.index(index, enum.kAccountTypeColumn_DebitField).data().toPyObject()
		currencySign = self.__accountsModel.index(index, enum.kAccountTypeColumn_CurrencySign).data().toPyObject()
		dateFormat = self.__accountsModel.index(index, enum.kAccountTypeColumn_DateFormat).data().toString()

		model = ImportModel(self)
		db.commit.connect(model.save)
		db.rollback.connect(model.reset)

		with utils.showWaitCursor():
			records = self.__processRecords(dateField, descriptionField, creditField, debitField, currencySign, dateFormat)
			model.loadRecords(records)

			proxy = QtGui.QSortFilterProxyModel(model)
			proxy.setSourceModel(model)
			proxy.setFilterKeyColumn(0)

			self.view.setModel(proxy)
			self.view.verticalHeader().hide()
			self.view.setSortingEnabled(True)
			self.view.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
			self.view.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
			self.view.horizontalHeader().setStretchLastSection(True)
			self.view.sortByColumn(0, QtCore.Qt.AscendingOrder)
			self.view.resizeColumnsToContents()
			self.view.selectionModel().selectionChanged.connect(self.__recordsSelected)
			self.importCancelButton.setEnabled(False)
			self.selectAllButton.setEnabled(bool(model.numRecordsToImport()))
			self.__setCounters()

		# Hide txDate colum if we don't need it
		for row in xrange(model.rowCount()):
			if model.index(row, enum.kImportColumn_TxDate).data().isValid():
				self.view.setColumnHidden(enum.kImportColumn_TxDate, False)
				break
		else:
			self.view.setColumnHidden(enum.kImportColumn_TxDate, True)

	def __processRecords(self, dateIdx, descriptionIdx, creditIdx, debitIdx, currencySign, dateFormat):
		""" Generator to decode the raw csv data according to the account configuration.
			Yields record containing verified data ready to be saved to database
		"""

		for filename, rawRecords in self.__rawData.iteritems():
			for lineno, rawdata in enumerate(rawRecords):

				if not rawdata:
					continue

				dateField = descField = txDate = debitField = creditField = None
				row = unicode_csv_reader([rawdata.data().decode('utf8')]).next()

				try:
					if max(dateIdx, descriptionIdx, creditIdx, debitIdx) > len(row) -1:
						raise DecoderError('Bad record')

					dateField  = self.__getDateField(row[dateIdx], dateFormat)
					descField  = row[descriptionIdx]
					txDate     = self.__getTransactionDate(row[descriptionIdx], dateField)

					if debitIdx == creditIdx:
						amount = self.__getAmountField(row[debitIdx])
						if amount is not None:
							# Use currency multiplier to ensure that credit is +ve (money in),
							# debit -ve (money out)
							amount *= currencySign

							if amount > 0.0:
								creditField = amount
							else:
								debitField = amount
					else:
						debitField = self.__getAmountField(row[debitIdx])
						creditField = self.__getAmountField(row[creditIdx])
						debitField = abs(debitField) * -1.0 if debitField else None
						creditField = abs(creditField) if creditField else None

					if not debitField and not creditField:
						raise DecoderError('No credit or debit found')

				except DecoderError, exc:
					error = '%s[%d]: %r' % (QtCore.QFileInfo(filename).fileName(), lineno, str(exc))
					yield (rawdata, dateField, descField, txDate, debitField, creditField, error,)

				except Exception, exc:
					QtGui.QMessageBox.critical(
						self, 'Import Error', str(exc),
						QtGui.QMessageBox.Ok)

				else:
					yield (rawdata, dateField, descField, txDate, debitField, creditField, None,)

	def __getDateField(self, field, dateFormat):
		""" Extract date field using supplied format 
		"""
		date = QtCore.QDate.fromString(field, dateFormat)

		if not date.isValid():
			raise DecoderError('Invalid date: %r' % field)

		return date

	def __getTransactionDate(self, field, dateField):
		""" Try and extract a transaction date from the description field.
			Value format are ddMMMyy hhmm, ddMMMyy and ddMMM. 
			When the year is not available (or 2 digits) then the value validated date field
			is used
		"""
		timeDate = None

		#Format is "23DEC09 1210"
		rx = QtCore.QRegExp('(\\d\\d[A-Z]{3}\\d\\d \\d{4})')
		if rx.indexIn(field) != -1:
			timeDate = QtCore.QDateTime.fromString (rx.cap(1), "ddMMMyy hhmm").addYears(100)

		if timeDate is None:
			# Format is "06NOV10"
			rx = QtCore.QRegExp('(\\d{2}[A-Z]{3}\\d{2})')
			if rx.indexIn(field) != -1:
				timeDate = QtCore.QDateTime.fromString (rx.cap(1), "ddMMMyy").addYears(100)

		# Format is " 06NOV" <- note the stupid leading blank space..
		if timeDate is None:
			rx = QtCore.QRegExp(' (\\d\\d[A-Z]{3})')
			if rx.indexIn(field) != -1:
				# Add the year from date field to the transaction date
				timeDate = QtCore.QDateTime.fromString (rx.cap(1) + dateField.toString("yyyy"), "ddMMMyyyy")

		if timeDate is not None and timeDate.isValid():
			return timeDate

	def __getAmountField(self, field):
		""" Extract and return amount (double). If a simple conversion doesn't
			succeed, then try and parse the string to remove any currency sign
			or other junk.

			Returns None if field does not contain valid double.
		"""

		# Get rid of commas from amount field and try and covert to double
		field = field.replace(',', '')
		value, ok = QtCore.QVariant(field).toDouble()

		if not ok:
			# Probably has currency sign - extract all valid currency characters
			match = re.search('([\d\-\.]+)', field)
			if match:
				value, ok = QtCore.QVariant(match.group(1)).toDouble()

		if ok:
			return value

		return None

	def __importCancelPressed(self):
		if self.__importInProgress:
			self.__cancelImport = True
		else:
			self.__importRecords()

	def __setCounters(self):
		model = self.view.model().sourceModel()
		self.errorsCounter.setNum(model.numBadRecords())
		self.importedCounter.setNum(model.numRecordsImported())
		self.toImportCounter.setNum(model.numRecordsToImport())

	def __recordsSelected(self):
		""" Enable button cancel when we have selection
		"""
		numSelected = len(self.view.selectionModel().selectedRows())
		self.selectedCounter.setNum(numSelected)
		self.importCancelButton.setEnabled(bool(numSelected))

	def __close(self):
		""" Exit with bool value to indicate if data was saved,
			but not if it's our initial model
		"""
		model = self.view.model()

		if isinstance(model, QtGui.QStandardItemModel):
			self.done(0)
		else:
			self.done(self.view.model().sourceModel().dataSaved)

	def __importRecords(self):
		""" Import selected rows to database
		"""
		index = self.accountTypeComboBox.currentIndex()
		accountId = self.__accountsModel.index(index, enum.kAccountTypeColumn_AccountTypeId).data().toPyObject()
		selectionModel = self.view.selectionModel()
		indexes = selectionModel.selectedRows()

		if len(indexes) == 0:
			return

		try:
			self.progressBar.setVisible(True)

			proxyModel = self.view.model()
			dataModel = proxyModel.sourceModel()
			self.progressBar.setValue(0)
			self.progressBar.setMaximum(len(indexes))
			self.view.clearSelection()

			self.__importInProgress = True
			self.closeButton.setEnabled(False)
			self.selectAllButton.setEnabled(False)
			self.importCancelButton.setText('Cancel')
			self.importCancelButton.setEnabled(True)

			# Wrap the import in a transaction
			with db.transaction():
				for num, index in enumerate(indexes, 1):
					dataModel.saveRecord(accountId, proxyModel.mapToSource(index))
					self.view.scrollTo(index, QtGui.QAbstractItemView.EnsureVisible)
					self.view.resizeColumnsToContents()
					self.__setCounters()
					QtCore.QCoreApplication.processEvents()
					self.progressBar.setValue(self.progressBar.value() +1)

					if self.__cancelImport:
						raise UserCancelledException

				if num:
					if QtGui.QMessageBox.question(
						self, 'Import', 'Imported %d records successfully' % num,
						QtGui.QMessageBox.Save|QtGui.QMessageBox.Cancel) != QtGui.QMessageBox.Save:
						# By raising here we will rollback the database transaction
						raise UserCancelledException

		except UserCancelledException:
			dataModel.reset()

		except Exception, exc:
			QtGui.QMessageBox.critical(self, 'Import Error', str(exc), QtGui.QMessageBox.Ok)

		finally:
			self.__cancelImport = False
			self.__importInProgress = False
			self.closeButton.setEnabled(True)
			self.importCancelButton.setText('Import')
			self.progressBar.setVisible(False)

			canImport = bool(dataModel.numRecordsToImport())
			self.importCancelButton.setEnabled(canImport)
			self.selectAllButton.setEnabled(canImport)


class SettingsDialog(Ui_Settings, QtGui.QDialog):
	def __init__(self, userId, parent=None):
		super(SettingsDialog, self).__init__(parent=parent)

		self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
		self.setupUi(self)

		self.saveButton.clicked.connect(self.saveSettings)
		self.revertButton.clicked.connect(self.cancelSettings)
		self.closeButton.clicked.connect(self.close)
		self.addButton.clicked.connect(self.addAccount)
		self.deleteButton.clicked.connect(self.deleteAccount)

		self.enableCommit(False)

		model = AccountEditModel(self)
		model.setTable('accounttypes')
		model.setEditStrategy(QtSql.QSqlTableModel.OnManualSubmit)
		model.select()

		self.view.setModel(model)
		self.view.setColumnHidden(enum.kAccountTypeColumn_AccountTypeId, True)
		self.view.verticalHeader().hide()
		self.view.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
		self.view.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
		self.view.sortByColumn(enum.kAccountTypeColumn_AccountName, QtCore.Qt.AscendingOrder)
		self.view.resizeColumnsToContents()
		self.view.horizontalHeader().setStretchLastSection(True)
		self.view.setItemDelegate(AccountDelegate(self))

		model.dataChanged.connect(self.view.clearSelection)
		model.dataChanged.connect(self.__dataChanged)
		model.beforeInsert.connect(self.validateNewAccount)

		self.model = model

	def __dataChanged(self, left, right):
		self.enableCommit(True)

	def validateNewAccount(self, record):
		try:
			if record.value(enum.kAccountTypeColumn_AccountName).toString().isEmpty():
				raise Exception('Account name cannot be empty')

			value, ok = record.value(enum.kAccountTypeColumn_DateField).toInt()
			if not ok or value < 0:
				raise Exception('Date field must be set (index of date field)')

			value, ok = record.value(enum.kAccountTypeColumn_CreditField).toInt()
			if not ok or value < 0:
				raise Exception('Credit field must be set (index of credit field)')

			value, ok = record.value(enum.kAccountTypeColumn_DebitField).toInt()
			if not ok or value < 0:
				raise Exception('Debit field must be set (index of debit field)')

			if record.value(enum.kAccountTypeColumn_DescriptionField).toString().isEmpty():
				raise Exception('Description field must be set (index of description field)')

			value, ok = record.value(enum.kAccountTypeColumn_CurrencySign).toInt()
			if not ok or value not in (1, -1):
				raise Exception('Currency sign value must be 1 or -1')

			if record.value(enum.kAccountTypeColumn_DateFormat).toString().isEmpty():
				raise Exception('"Date format cannot be empty (eg "dd/MM/yyyy")')

		except Exception, err:
			QtGui.QMessageBox.critical(self, 'Account failed', str(err), QtGui.QMessageBox.Ok)
			# Trash the bad record
			record.clear()

	def saveSettings(self):

		if not self.model.submitAll() and self.model.lastError().isValid():
			# If we've cleared the record from validateNewAccount() then the database error
			# will be empty. No need to issue a second error message
			if self.m_model.lastError().databaseText():
				QtGui.QMessageBox.critical(self, 'Error saving data', self.model.lastError().text(), QtGui.QMessageBox.Ok)
			self.model.revertAll()

		self.enableCommit(False)

	def cancelSettings(self): 
		self.model.revertAll()
		self.enableCommit(False)

	def enableCommit(self, enable):
		self.saveButton.setEnabled(enable)
		self.revertButton.setEnabled(enable)

	def addAccount(self):
		rowCount = self.model.rowCount()
		self.model.insertRow(rowCount)

		for column in xrange(1, self.model.columnCount()):
			index = self.model.index(rowCount, column)
			self.model.setData(index, QtCore.QVariant(), QtCore.Qt.EditRole)

		index = self.model.index(rowCount, enum.kAccountTypeColumn_AccountName)
		self.view.setCurrentIndex(index)
		self.view.edit(index)

	def deleteAccount(self):
		for index in self.view.selectionModel().selectedRows():
			if index.isValid():
				self.model.removeRows(index.row(), 1, QtCore.QModelIndex())

		self.view.clearSelection()
		self.enableCommit(True)


class LoginDialog(Ui_Login, QtGui.QDialog):
	def __init__(self, parent=None):
		super(LoginDialog, self).__init__(parent=parent)

		self.setAttribute(QtCore.Qt.WA_DeleteOnClose)
		self.setupUi(self)

		self.passwordEdit.setEchoMode(QtGui.QLineEdit.Password)

		self.connectionButton.clicked.connect(self.activateConnection)
		self.closeButton.clicked.connect(self.reject)

		self.hostnameEdit.setText(db.hostname)
		self.databaseEdit.setText(db.database)
		self.usernameEdit.setText(db.username)
		self.passwordEdit.setText(db.password)
		self.portSpinBox.setValue(db.port)

	def activateConnection(self):
		db.database = self.databaseEdit.text()
		db.hostname = self.hostnameEdit.text()
		db.username = self.usernameEdit.text()
		db.password = self.passwordEdit.text()
		db.port = self.portSpinBox.value()

		try:
			with utils.showWaitCursor():
				db.connect()
		except DatabaseNotInitialisedException:
			if QtGui.QMessageBox.question(
					self, 'Database', 
					'Database %s is empty, do you want to initialise it?' % db.database, 
					QtGui.QMessageBox.Yes|QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
				try:
					db.initialise()
				except ConnectionException, err:
					QtGui.QMessageBox.critical(self, 'Database ', str(err))
				else:
					QtGui.QMessageBox.information(self, 'Database', 'Database initialised successfully')
			else:
				return
		except ConnectionException, err:
			QtGui.QMessageBox.warning(self, 'Database', 'Failed to connect: %s' % str(err))
		else:
			self.accept()



import collections
from contextlib  import contextmanager
from PyQt4 import QtGui, QtCore

def flattenArgs(items):
	""" Generator to expand a sequence of items recursively.
		Eg (1, [2, 3], ["a", "b", [10]]) -> (1, 2, 3, "a", "b", 10)
	"""
	for item in items:
		if isinstance(item, collections.Iterable) and not isinstance(item, basestring):
			for sub in flattenArgs(item):
				yield sub
		else:
			yield item

@contextmanager
def signalsBlocked(*args):
	""" Block signals with context manager
		Args: 
			args: widget or widgets
	"""
	states = [(target, target.signalsBlocked()) for target in flattenArgs(args) if target is not None]
	try:
		for target, _ in states:
			target.blockSignals(True)
		yield
	finally:
		for target, blockedState in states:
			target.blockSignals(blockedState)

def showWaitCursorDecorator(f):
	""" Decorator for display a wait cursor whilst in a slow function
	"""
	def new_f(*args, **kwargs):
		QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
		try:
			return f(*args, **kwargs)
		finally:
			QtGui.QApplication.restoreOverrideCursor()
	new_f.__name__ = f.__name__
	new_f.__doc__  = f.__doc__
	return new_f

@contextmanager
def showWaitCursor():
	""" Context manager utility for showing wait cursor for a code block
	"""
	try:
		QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor)
		yield
	finally:
		QtGui.QApplication.restoreOverrideCursor()



from PyQt4 import QtGui, QtCore
import enum

class AccountDelegate(QtGui.QItemDelegate):
	def __init__(self, parent=None):
		super(AccountDelegate, self).__init__(parent=parent)

	def createEditor(self, parent, option, index):

		lineEdit = QtGui.QLineEdit(parent=parent)

		if index.column() in (
						enum.kAccountTypeColumn_DateField,
						enum.kAccountTypeColumn_DescriptionField,
						enum.kAccountTypeColumn_CreditField,
						enum.kAccountTypeColumn_DebitField):
			lineEdit.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp('[0-9]+'), lineEdit))

		elif index.column() == enum.kAccountTypeColumn_CurrencySign:
			lineEdit.setValidator(QtGui.QRegExpValidator(QtCore.QRegExp('-1|1')))

		return lineEdit

	def setModelData(self, editor, model, index):

		if not index.isValid():
			return

		if editor:
			model.setData(index, editor.text())

	def updateEditorGeometry(self, editor, option, index):
		editor.setGeometry(option.rect)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/settings.ui'
#
# Created: Sat Jun 22 12:53:39 2013
#      by: PyQt4 UI code generator 4.10
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_Settings(object):
    def setupUi(self, Settings):
        Settings.setObjectName(_fromUtf8("Settings"))
        Settings.resize(708, 364)
        self.gridLayout_2 = QtGui.QGridLayout(Settings)
        self.gridLayout_2.setObjectName(_fromUtf8("gridLayout_2"))
        spacerItem = QtGui.QSpacerItem(23, 28, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 0, 1, 1, 1)
        self.addButton = QtGui.QToolButton(Settings)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/add.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.addButton.setIcon(icon)
        self.addButton.setObjectName(_fromUtf8("addButton"))
        self.gridLayout_2.addWidget(self.addButton, 1, 1, 1, 1)
        self.frame = QtGui.QFrame(Settings)
        self.frame.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtGui.QFrame.Raised)
        self.frame.setObjectName(_fromUtf8("frame"))
        self.gridLayout = QtGui.QGridLayout(self.frame)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.view = QtGui.QTableView(self.frame)
        self.view.setObjectName(_fromUtf8("view"))
        self.gridLayout.addWidget(self.view, 0, 0, 1, 1)
        self.gridLayout_2.addWidget(self.frame, 0, 0, 4, 1)
        self.deleteButton = QtGui.QToolButton(Settings)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/icons/delete.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.deleteButton.setIcon(icon1)
        self.deleteButton.setObjectName(_fromUtf8("deleteButton"))
        self.gridLayout_2.addWidget(self.deleteButton, 2, 1, 1, 1)
        spacerItem1 = QtGui.QSpacerItem(23, 217, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1, 3, 1, 1, 1)
        self.widget_4 = QtGui.QWidget(Settings)
        self.widget_4.setObjectName(_fromUtf8("widget_4"))
        self.hboxlayout = QtGui.QHBoxLayout(self.widget_4)
        self.hboxlayout.setContentsMargins(0, -1, 0, -1)
        self.hboxlayout.setObjectName(_fromUtf8("hboxlayout"))
        spacerItem2 = QtGui.QSpacerItem(136, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.hboxlayout.addItem(spacerItem2)
        self.saveButton = QtGui.QPushButton(self.widget_4)
        self.saveButton.setObjectName(_fromUtf8("saveButton"))
        self.hboxlayout.addWidget(self.saveButton)
        self.revertButton = QtGui.QPushButton(self.widget_4)
        self.revertButton.setObjectName(_fromUtf8("revertButton"))
        self.hboxlayout.addWidget(self.revertButton)
        self.closeButton = QtGui.QPushButton(self.widget_4)
        self.closeButton.setObjectName(_fromUtf8("closeButton"))
        self.hboxlayout.addWidget(self.closeButton)
        self.gridLayout_2.addWidget(self.widget_4, 4, 0, 1, 1)

        self.retranslateUi(Settings)
        QtCore.QMetaObject.connectSlotsByName(Settings)
        Settings.setTabOrder(self.saveButton, self.revertButton)
        Settings.setTabOrder(self.revertButton, self.closeButton)
        Settings.setTabOrder(self.closeButton, self.addButton)
        Settings.setTabOrder(self.addButton, self.deleteButton)

    def retranslateUi(self, Settings):
        Settings.setWindowTitle(_translate("Settings", "Options", None))
        self.addButton.setToolTip(_translate("Settings", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Add code</p></body></html>", None))
        self.addButton.setText(_translate("Settings", "...", None))
        self.deleteButton.setToolTip(_translate("Settings", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Delete code</p></body></html>", None))
        self.deleteButton.setText(_translate("Settings", "...", None))
        self.saveButton.setText(_translate("Settings", "&Save", None))
        self.revertButton.setText(_translate("Settings", "&Revert", None))
        self.closeButton.setText(_translate("Settings", "&Close", None))

import pydosh_rc



import sys
from PyQt4 import QtGui, QtCore
from dialogs import LoginDialog
from main import PydoshWindow
import stylesheet
#QtCore.pyqtRemoveInputHook()
import pydosh_rc

def main():

	app = QtGui.QApplication(sys.argv)
	app.setWindowIcon(QtGui.QIcon(":/icons/pydosh.png"))

	QtCore.QCoreApplication.setApplicationName("pydosh")
	QtCore.QCoreApplication.setOrganizationName("innerhippy")
	QtCore.QCoreApplication.setOrganizationDomain("innerhippy.com")

	stylesheet.setStylesheet()

	loginDialog = LoginDialog()
	loginDialog.show()
	loginDialog.raise_()

	if loginDialog.exec_():
		window = PydoshWindow()
		window.show()
		return app.exec_()

	return -1

if __name__ == '__main__':
	sys.exit(main())



from PyQt4 import QtGui, QtCore

__styleNames = ['Default', 'Dark']

def styleSheetNames():
	return __styleNames

def setStylesheet(name=None):
	settings = QtCore.QSettings()

	if name is None:
		name = settings.value("options/stylesheet", 'Default').toString()

	styleSheetFile = QtCore.QFile(':/style/%s' % name)
	styleSheetFile.open(QtCore.QIODevice.ReadOnly)
	styleSheet = QtCore.QString(styleSheetFile.readAll())

	QtGui.QApplication.instance().setStyleSheet(styleSheet)
	settings.setValue('options/stylesheet', name)
